@@ -1,4 +1,4 @@
-# Copyright (c) 2012-2013 Roger Light <roger@atchoo.org>
+# Copyright (c) 2012-2014 Roger Light <roger@atchoo.org>
 #
 # All rights reserved. This program and the accompanying materials
 # are made available under the terms of the Eclipse Public License v1.0
@@ -47,13 +47,18 @@ else:
 
 VERSION_MAJOR=0
 VERSION_MINOR=4
-VERSION_REVISION=0
+VERSION_REVISION=93
 VERSION_NUMBER=(VERSION_MAJOR*1000000+VERSION_MINOR*1000+VERSION_REVISION)
 
+MQTTv31 = 3
+MQTTv311 = 4
+
 if sys.version_info[0] < 3:
-    PROTOCOL_NAME = "MQIsdp"
+    PROTOCOL_NAMEv31 = "MQIsdp"
+    PROTOCOL_NAMEv311 = "MQTT"
 else:
-    PROTOCOL_NAME = b"MQIsdp"
+    PROTOCOL_NAMEv31 = b"MQIsdp"
+    PROTOCOL_NAMEv311 = b"MQTT"
 
 PROTOCOL_VERSION = 3
 
@@ -119,6 +124,10 @@ MQTT_ERR_ACL_DENIED = 12
 MQTT_ERR_UNKNOWN = 13
 MQTT_ERR_ERRNO = 14
 
+if sys.version_info[0] < 3:
+    sockpair_data = "0"
+else:
+    sockpair_data = b"0"
 
 def error_string(mqtt_errno):
     """Return the error string associated with an mqtt error number."""
@@ -235,6 +244,28 @@ def topic_matches_sub(sub, topic):
     return result
 
 
+def _socketpair_compat():
+    """TCP/IP socketpair including Windows support"""
+    listensock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, socket.IPPROTO_IP)
+    listensock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+    listensock.bind(("localhost", 0))
+    listensock.listen(1)
+
+    iface, port = listensock.getsockname()
+    sock1 = socket.socket(socket.AF_INET, socket.SOCK_STREAM, socket.IPPROTO_IP)
+    sock1.setblocking(0)
+    try:
+        sock1.connect(("localhost", port))
+    except socket.error as err:
+        (msg) = err
+        if msg.errno != errno.EINPROGRESS:
+            raise
+    sock2, address = listensock.accept()
+    sock2.setblocking(0)
+    listensock.close()
+    return (sock1, sock2)
+
+
 class MQTTMessage:
     """ This is a class that describes an incoming message. It is passed to the
     on_message callback as the message parameter.
@@ -259,7 +290,7 @@ class MQTTMessage:
 
 
 class Client(object):
-    """MQTT version 3.1 client class.
+    """MQTT version 3.1/3.1.1 client class.
 
     This is the main class for use communicating with an MQTT broker.
 
@@ -341,7 +372,7 @@ class Client(object):
       MQTT_LOG_ERR, and MQTT_LOG_DEBUG. The message itself is in buf.
 
     """
-    def __init__(self, client_id="", clean_session=True, userdata=None):
+    def __init__(self, client_id="", clean_session=True, userdata=None, protocol=MQTTv31):
         """client_id is the unique client id string used when connecting to the
         broker. If client_id is zero length or None, then one will be randomly
         generated. In this case, clean_session must be True. If this is not the
@@ -363,8 +394,10 @@ class Client(object):
         if not clean_session and (client_id == "" or client_id is None):
             raise ValueError('A client id must be provided if clean session is False.')
 
+        self._protocol = protocol
         self._userdata = userdata
         self._sock = None
+        self._sockpairR, self._sockpairW = _socketpair_compat()
         self._keepalive = 60
         self._message_retry = 20
         self._last_retry_check = 0
@@ -441,6 +474,13 @@ class Client(object):
         elif self._sock:
             self._sock.close()
             self._sock = None
+        if self._sockpairR:
+            self._sockpairR.close()
+            self._sockpairR = None
+        if self._sockpairW:
+            self._sockpairW.close()
+            self._sockpairW = None
+
         self.__init__(client_id, clean_session, userdata)
 
     def tls_set(self, ca_certs, certfile=None, keyfile=None, cert_reqs=cert_reqs, tls_version=tls_version, ciphers=None):
@@ -728,7 +768,9 @@ class Client(object):
         self._out_packet_mutex.release()
         self._current_out_packet_mutex.release()
 
-        rlist = [self.socket()]
+        # sockpairR is used to break out of select() before the timeout, on a
+        # call to publish() etc.
+        rlist = [self.socket(), self._sockpairR]
         try:
             socklist = select.select(rlist, wlist, [], timeout)
         except TypeError:
@@ -740,6 +782,17 @@ class Client(object):
             if rc or (self._ssl is None and self._sock is None):
                 return rc
 
+        if self._sockpairR in socklist[0]:
+            # Stimulate output write even though we didn't ask for it, because
+            # at that point the publish or other command wasn't present.
+            socklist[1].insert(0, self.socket())
+            # Clear sockpairR - only ever a single byte written.
+            try:
+                self._sockpairR.recv(1)
+            except socket.error as err:
+                if err.errno != errno.EAGAIN:
+                    raise
+
         if self.socket() in socklist[1]:
             rc = self.loop_write(max_packets)
             if rc or (self._ssl is None and self._sock is None):
@@ -1616,7 +1669,14 @@ class Client(object):
         packet = bytearray()
         packet.extend(struct.pack("!B", command))
         self._pack_remaining_length(packet, remaining_length)
-        packet.extend(struct.pack("!H6sBBH", len(PROTOCOL_NAME), PROTOCOL_NAME, PROTOCOL_VERSION, connect_flags, keepalive))
+        if self._protocol == MQTTv31:
+            protocol = PROTOCOL_NAMEv31
+            proto_ver = 3
+        else:
+            protocol = PROTOCOL_NAMEv311
+            proto_ver = 4
+
+        packet.extend(struct.pack("!H"+str(len(protocol))+"sBBH", len(protocol), protocol, proto_ver, connect_flags, keepalive))
 
         self._pack_str16(packet, self._client_id)
 
@@ -1739,6 +1799,14 @@ class Client(object):
             self._current_out_packet_mutex.release()
         self._out_packet_mutex.release()
 
+        # Write a single byte to sockpairW (connected to sockpairR) to break
+        # out of select() if in threaded mode.
+        try:
+            self._sockpairW.send(sockpair_data)
+        except socket.error as err:
+            if err.errno != errno.EAGAIN:
+                raise
+
         if not self._in_callback and self._thread is None:
             return self.loop_write()
         else:
